/*
 * Copyright (c) 2002-2003 WIPRO TECHNOLOGIES 
 * Developed for LSB-PAM project by WIPRO TECHNOLOGIES
 */

/*
 * Project: LSB-PAM
 *
 * File: tset/LSB.pam/testcases/pam_acct_mgmt/pam_acct_mgmt.c
 * This file contains the test cases for testing the interface
 * pam_acct_mgmt.
 *
 * Author(s) : Rohit R <rohit.raveendran@wipro.com>, WIPRO TECHNOLOGIES
 *             Laavanya K R <laavanya.gopalan@wipro.com>, WIPRO TECHNOLOGIES
 *
 * Modifications:
 *
 */

#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <security/pam_appl.h>
#include <security/pam_misc.h>

#include <std.h>
#include <tet_api.h>

/*******************************************************************/

#define NUM_TESTS   5 /* To be updated when number of tests are modified */

#define INIT_PASS    0
#define INIT_FAIL    1

private void tc1();
private void tc2();
private void tc3();
private void tc4();
private void tc5();

private void prepare_tests();
private void clean_tests();

public  void (*tet_startup)() = prepare_tests;
public  void (*tet_cleanup)() = clean_tests;

private char *conf_file;

private char *user;
private char *passwd;
private char *input_passwd;

private char *test_user1;
private char *test_passwd1;

private char *test_user2;
private char *test_passwd2;

private char echo[256];

private pam_handle_t *pamh;
private struct pam_conv pamc;

public struct tet_testlist tet_testlist[] = {
        { tc1, 1 },
        { tc2, 2 },
        { tc3, 3 },
        { tc4, 4 },
        { tc5, 5 },
        { NULL, 0 }
};

private int login_conv(int num_msg,
                       const struct pam_message **msg,
                       struct pam_response **response,
                       void *appdata_ptr);


private void delete_tests(char *msg);

/*******************************************************************/

private void
prepare_tests()
{
    int    retval;


    tet_infoline(" starting tests pam_acct_mgmt ");

    pamh = NULL;

    conf_file = tet_getvar("LSB_PAM_CONF_FILE");
    if (conf_file == NULL) {
        delete_tests(" tetexec.cfg parameter LSB_PAM_CONF_FILE unset");
        return;
    }

    user = tet_getvar("LSB_PAM_USER");
    if (user == NULL) {
        delete_tests(" tetexec.cfg parameter LSB_PAM_USER unset");
        return;
    }

    passwd = tet_getvar("LSB_PAM_USER_PASSWD");
    if (passwd == NULL) {
        delete_tests(" tetexec.cfg parameter LSB_PAM_USER_PASSWD unset");
        return;
    }


    test_user1 = tet_getvar("LSB_PAM_TEST_USER1");
    if (test_user1 == NULL) {
        delete_tests(" tetexec.cfg parameter LSB_PAM_TEST_USER1 unset");
        return;
    }

    test_passwd1 = tet_getvar("LSB_PAM_TEST_USER1_PASSWD");
    if (test_passwd1 == NULL) {
        delete_tests(" tetexec.cfg parameter LSB_PAM_TEST_USER1_PASSWD unset");
        return;
    }

    test_user2 = tet_getvar("LSB_PAM_TEST_USER2");
    if (test_user2 == NULL) {
        delete_tests(" tetexec.cfg parameter LSB_PAM_TEST_USER2 unset");
        return;
    }

    test_passwd2 = tet_getvar("LSB_PAM_TEST_USER2_PASSWD");
    if (test_passwd2 == NULL) {
        delete_tests(" tetexec.cfg parameter LSB_PAM_TEST_USER2_PASSWD unset");
        return;
    }

    return;
}


int initialise(char *pam_user, char *pam_passwd)
{
    int    retval;

    if (pamh) {
        pam_end(pamh, 0);
    }

    pamh = NULL;
    pamc.conv = &login_conv;
    pamc.appdata_ptr = NULL;

    retval = pam_start(conf_file, pam_user, &pamc, &pamh);

    if (retval != PAM_SUCCESS) {
        tet_printf(" pam_start returned %s when expected to succeed",
                                pam_strerror(NULL, retval));
        return INIT_FAIL;
    }

    input_passwd = pam_passwd;
    retval = pam_authenticate(pamh, 0);

    if (retval != PAM_SUCCESS) {
        tet_printf(" pam_authenticate returned %s when expected to succeed",
                                pam_strerror(NULL, retval));
        tet_printf(" uid : %d", getuid());
        tet_printf(" euid : %d", geteuid());
        return INIT_FAIL;
    }

    return INIT_PASS;
}


private void
clean_tests()
{
    if (pamh) {
        pam_end(pamh,0);
    }

    tet_infoline(" end of tests pam_acct_mgmt ");
    return;
}


/*
 *    tc1():
 *        This test case checks the interface pam_acct_mgmt for normal user.
 *        The expected return value is PAM_SUCCESS.
 */

private void
tc1()
{
    int    retval;

    retval = initialise(user, passwd);

    if (retval == INIT_FAIL) {
        tet_infoline(" initialise failed ");
        tet_result(TET_FAIL);
        return;
    }
    
    retval = pam_acct_mgmt(pamh, 0);

    if (retval != PAM_SUCCESS) {
        sprintf(echo," pam_acct_mgmt returned %s when expected to succeed",
                                pam_strerror(pamh, retval));
        delete_tests(echo);

        tet_result(TET_FAIL);
        return;
    }

    pam_end(pamh,0);
    pamh = NULL;

    tet_result(TET_PASS);
    return;
}


/*
 *    tc2():
 *        This test case checks the interface pam_acct_mgmt with an
 *        empty user name. The expected return value is PAM_USER_UNKNOWN.
 */

private void
tc2()
{
    int     retval;
    char    *value = NULL;
    char    *orig_value = NULL;


    retval = initialise(user, passwd);

    if (retval == INIT_FAIL) {
        tet_infoline(" initialise failed ");
        tet_result(TET_FAIL);
        return;
    }
    
    retval = pam_get_item(pamh, PAM_USER, (const void **)&value);

    if (retval == PAM_SUCCESS ) {

        if (value != NULL) {

            orig_value = strdup(value);
            if (orig_value) {

                retval = pam_set_item(pamh, PAM_USER, "");
                if (retval != PAM_SUCCESS ) {

                    tet_infoline(" pam_acct_mgmt: pam_set_item failed ");
                    tet_result(TET_FAIL);
                    free(orig_value);
                    return;
                }

            } else {
                tet_infoline(" pam_acct_mgmt: strdup returned NULL ");
                tet_result(TET_FAIL);
                return;
            }

        } else {
            tet_infoline(" pam_acct_mgmt: pam_get_item returned NULL ");
            tet_result(TET_FAIL);
            return;
        }

    } else {
        tet_infoline(" pam_acct_mgmt: pam_get_item failed ");
        tet_result(TET_FAIL);
        return;
    }

    retval = pam_acct_mgmt(pamh, 0);

    pam_set_item(pamh, PAM_USER, orig_value); /* What to do if this set fails */
    free(orig_value);

    if (retval != PAM_USER_UNKNOWN) {
        tet_infoline(" pam_acct_mgmt expected to return PAM_USER_UNKNOWN for");
        tet_printf(" NULL username. It returned %s",
                                pam_strerror(pamh, retval));

        tet_result(TET_FAIL);
        return;
    }

    pam_end(pamh, 0);
    pamh = NULL;

    tet_result(TET_PASS);
    return;
}


/*
 *    tc3():
 *        This test case checks the interface pam_acct_mgmt with
 *        a NULL pam handle. The return value should not be PAM_SUCCESS.
 */

private void
tc3()
{
    int    retval;

    retval = initialise(user, passwd);

    if (retval == INIT_FAIL) {
        tet_infoline(" initialise failed ");
        tet_result(TET_FAIL);
        return;
    }
    
    retval = pam_acct_mgmt(NULL, 0);

    if (retval == PAM_SUCCESS) {
        tet_infoline(" pam_acct_mgmt returned PAM_SUCCESS with NULL pamh");
        tet_infoline(" It was expected not to succeed.");
        tet_result(TET_FAIL);
        return;
    }

    pam_end(pamh, 0);
    pamh = NULL;

    tet_result(TET_PASS);
    return;
}


/*
 *    tc4():
 *        This test case checks the interface pam_acct_mgmt with
 *        a test user whose password has expired. The expected return 
 *        value is PAM_NEW_AUTHTOK_REQD.
 */

private void
tc4()
{
    int    retval;

    retval = initialise(test_user1, test_passwd1);

    if (retval == INIT_FAIL) {
        tet_infoline(" initialise failed ");
        tet_result(TET_FAIL);
        return;
    }
    
    retval = pam_acct_mgmt(pamh, 0);

    if (retval != PAM_NEW_AUTHTOK_REQD) {
        tet_infoline(" pam_acct_mgmt: expected to return PAM_NEW_AUTHTOK_REQD");
        tet_printf(" It returned %s.", pam_strerror(pamh, retval));
        tet_result(TET_FAIL);
        return;
    }

    pam_end(pamh, 0);
    pamh = NULL;

    tet_result(TET_PASS);
    return;
}


/*
 *    tc5():
 *        This test case checks the interface pam_acct_mgmt with
 *        a test user whose account has expired. The expected return 
 *        value is PAM_ACCT_EXPIRED.
 */

private void
tc5()
{
    int    retval;

    retval = initialise(test_user2, test_passwd2);

    if (retval == INIT_FAIL) {
        tet_infoline(" initialise failed ");
        tet_result(TET_FAIL);
        return;
    }
    
    retval = pam_acct_mgmt(pamh, 0);

    if (retval != PAM_ACCT_EXPIRED) {
        tet_infoline(" pam_acct_mgmt: expected to return PAM_ACCT_EXPIRED");
        tet_printf(" It returned %s.", pam_strerror(pamh, retval));
        tet_result(TET_FAIL);
        return;
    }

    pam_end(pamh, 0);
    pamh = NULL;

    tet_result(TET_PASS);
    return;
}


/*
 *   login_conv():
 *        This routine is called to get the username and password
 *        required for authenticating the user. Currently the
 *        username and password are got from the configuration files
 *        to avoid user intervention.
 */

private int
login_conv(int num_msg,
           const struct pam_message **m,
           struct pam_response **r,
           void *appdata_ptr)
{
    struct  pam_response *reply;
    int     count;
    char    buf[PAM_MAX_RESP_SIZE];
    char    *temp;

    reply = (struct pam_response *)
                calloc(num_msg, sizeof(struct pam_response));

    if (reply == NULL) {
        return PAM_BUF_ERR;
    }

    for (count = 0; count < num_msg; count++) {
        reply[count].resp_retcode = 0;
        reply[count].resp = NULL;

        switch (m[count]->msg_style) {

            case PAM_PROMPT_ECHO_OFF:

                reply[count].resp = strdup(input_passwd);
                break;

            case PAM_PROMPT_ECHO_ON:

                reply[count].resp = strdup(user);
                break;

            default:

                break;
        }
    }

    *r = reply;
    return PAM_SUCCESS;
}



/*
 *   delete_tests():
 *        This routine is called to stop execution of the test cases
 *        in case of initialization errors.
 */

private void
delete_tests(char *msg)
{
    int    count;

    for (count = 1; count <= NUM_TESTS; count++) {
        tet_delete(count, msg);
    }

    return;
}

