/*
 * Copyright (c) 2002-2003 WIPRO TECHNOLOGIES 
 * Developed for LSB-PAM project by WIPRO TECHNOLOGIES
 */

/*
 * Project: LSB-PAM
 *
 * File: tset/LSB.pam/testcases/pam_authenticate/pam_authenticate.c
 * This file contains the test cases for testing the interface
 * pam_authenticate.
 *
 * Author(s) : Rohit R <rohit.raveendran@wipro.com>, WIPRO TECHNOLOGIES
 *             Laavanya K R <laavanya.gopalan@wipro.com>, WIPRO TECHNOLOGIES
 *
 * Modifications:
 *
 */


#include <stdio.h>
#include <security/pam_appl.h>
#include <security/pam_misc.h>

#include <std.h>
#include <tet_api.h>

/*******************************************************************/

#define NUM_TESTS       6 /* To be updated when number of tests are modified */
#define MAXIMUM_TRIES  20 /* Assuming PAM_MAXTRIES would occur within
                             specified tries */
#define INIT_PASS       0
#define INIT_FAIL       1

private void tc1();
private void tc2();
private void tc3();
private void tc4();
private void tc5();
private void tc6();

private void prepare_tests();
private int initialise();
private void clean_tests();
private void cleanup();

public  void (*tet_startup)() = prepare_tests;
public  void (*tet_cleanup)() = clean_tests;

private char *conf_file;
private char *user;
private char *passwd;
private char *input_passwd;

private char echo[256];

private pam_handle_t *pamh;
private struct pam_conv pamc;

public  struct tet_testlist tet_testlist[] = {
        { tc1, 1 },
        { tc2, 2 },
        { tc3, 3 },
        { tc4, 4 },
        { tc5, 5 },
        { tc6, 6 },
        { NULL, 0 }
};

private int login_conv(int num_msg,
                       const struct pam_message **msg,
                       struct pam_response **response,
                       void *appdata_ptr);


private void delete_tests(char *msg);

/*******************************************************************/

private void
prepare_tests()
{
    int    retval;


    tet_infoline(" starting tests pam_authenticate ");

    pamh = NULL;
    pamc.conv = &login_conv;
    pamc.appdata_ptr = NULL;

    conf_file = tet_getvar("LSB_PAM_CONF_FILE");

    if (conf_file == NULL) {
        delete_tests(" tetexec.cfg parameter LSB_PAM_CONF_FILE unset");
        return;
    }

    user = tet_getvar("LSB_PAM_USER");

    if (user == NULL) {
        delete_tests(" tetexec.cfg parameter LSB_PAM_USER unset");
        return;
    }

    passwd = tet_getvar("LSB_PAM_USER_PASSWD");

    if (passwd == NULL) {
        delete_tests(" tetexec.cfg parameter LSB_PAM_USER_PASSWD unset");
        return;
    }

    if (INIT_FAIL == initialise()) {
        delete_tests(echo);
    }

    return;
}

private int
initialise()
{
    int    retval;

    retval = pam_start(conf_file, user, &pamc, &pamh);

    if (retval != PAM_SUCCESS) {
        sprintf(echo," pam_start returned %s when expected to succeed",
                                pam_strerror(NULL, retval));
        return INIT_FAIL;
    }

    return INIT_PASS;
}


private void
clean_tests()
{

    cleanup();
    tet_infoline(" end of tests pam_authenticate ");
    return;
}

private void
cleanup()
{
    if (pamh) {
        pam_end(pamh,0);
    }
}


/*
 *    tc1():
 *        This test case checks the interface pam_authenticate.
 *        The expected return value is PAM_SUCCESS.
 */

private void
tc1()
{
    int    retval;

    input_passwd = passwd;
    retval = pam_authenticate(pamh, 0);

    if (retval != PAM_SUCCESS) {

        sprintf(echo," pam_authenticate returned %s when expected to succeed",
                                pam_strerror(pamh, retval));
        delete_tests(echo);

        tet_result(TET_FAIL);
        return;
    }

    tet_result(TET_PASS);
    return;
}


/*
 *    tc2():
 *        This test case checks the interface pam_authenticate with
 *        wrong password. Expected return value is PAM_AUTH_ERR.
 */

private void
tc2()
{
    int    retval;
    char   *str = "11"; /* String appended to get invalid passwd */

    input_passwd = calloc(1, strlen(passwd) + strlen(str) + 5);

    if (input_passwd == NULL) {
        tet_infoline(" calloc failed : returning from test case ");
        tet_result(TET_FAIL);
        return;
    }

    strncpy(input_passwd, str, strlen(str));
    strncat(input_passwd, passwd, strlen(passwd));

    retval = pam_authenticate(pamh, 0);
    free(input_passwd);

    if (retval != PAM_AUTH_ERR) {
        tet_printf(" pam auth returned %s when expected to return PAM_AUTH_ERR",                                pam_strerror(pamh, retval));

        tet_result(TET_FAIL);
        return;
    }

    tet_result(TET_PASS);
    return;
}


/*
 *    tc3():
 *        This test case checks the interface pam_authenticate with
 *        flag PAM_DISALLOW_NULL_AUTHTOK and
 *
 *        1. correct password : expected return value is PAM_SUCCESS
 *        2. empty password : expected return value is PAM_AUTH_ERR.
 */

private void
tc3()
{
    int    retval;

    input_passwd = passwd;

    retval = pam_authenticate(pamh, PAM_DISALLOW_NULL_AUTHTOK);

    if (retval != PAM_SUCCESS) {
        tet_printf(" pam_authenticate returned %s when expected to succeed",
                                pam_strerror(pamh, retval));

        tet_result(TET_FAIL);
        return;
    }
	
    input_passwd = "\0";
    
    retval = pam_authenticate(pamh, PAM_DISALLOW_NULL_AUTHTOK);

    if (retval != PAM_AUTH_ERR) {
        tet_printf(" pam auth returned %s when expected to return PAM_AUTH_ERR",
                                pam_strerror(pamh, retval));

        tet_result(TET_FAIL);
        return;
    }

    tet_result(TET_PASS);
    return;
}


/*
 *    tc4():
 *        This test case checks the interface pam_authenticate with
 *        a NULL pam handle. The return value should not be PAM_SUCCESS.
 */

private void
tc4()
{
    int    retval;

    input_passwd = passwd;
    retval = pam_authenticate(NULL, 0);

    if (retval == PAM_SUCCESS) {

        tet_infoline(" pam auth returned PAM_SUCCESS, when expected not to succeed");
        tet_result(TET_FAIL);
        return;
    }

    tet_result(TET_PASS);
    return;
}

/*
 *    tc5():
 *        This test case checks the interface pam_authenticate with
 *        wrong password till the return value is PAM_MAXTRIES.
 */


private void
tc5()
{
    int    retval;
    int    count = MAXIMUM_TRIES;
    int    result = TET_FAIL;
    char   *str = "11";

/* This test has been disabled because pam modules
	 are not required to enforce PAM_MAXTRIES and we cannot
	 guarantee that the module tested by side effect does so */
    tet_result(TET_NOTINUSE);
		return;

    input_passwd = calloc(1, strlen(passwd) + strlen(str) + 5);

    if (input_passwd == NULL) {
        tet_infoline(" calloc failed : returning from test case ");
        tet_result(TET_FAIL);
        return;
    }

    strncpy(input_passwd, str, strlen(str));
    strncat(input_passwd, passwd, strlen(passwd));

    while (count) {
        retval = pam_authenticate(pamh, 0);

        if (retval == PAM_MAXTRIES ) {
            result = TET_PASS;
            break;
        }

        count--;
    }

    free(input_passwd);
    tet_result(result);
    return;
}


/*
 *    tc6():
 *        This test case checks the interface pam_authenticate with invalid
 *        user. The expected return value is PAM_USER_UNKNOWN.
 */

private void
tc6()
{
    int    retval;
    char    *value = NULL;
    char    *orig_value = NULL;

    cleanup();

    if (INIT_FAIL == initialise()) {
        tet_infoline(" pam_start failed ");
        tet_result(TET_FAIL);
        return;
    }

    retval = pam_get_item(pamh, PAM_USER, (const void **)&value);

    if (retval == PAM_SUCCESS ) {

        if (value != NULL) {

            orig_value = strdup(value);
            if (orig_value) {

                retval = pam_set_item(pamh, PAM_USER, "1234");
                if (retval != PAM_SUCCESS ) {

                    tet_infoline(" pam_set_item failed ");
                    tet_result(TET_FAIL);
                    free(orig_value);
                    return;
                }

            } else {
                tet_infoline(" strdup returned NULL ");
                tet_result(TET_FAIL);
                return;
            }

        } else {
            tet_infoline(" pam_get_item returned NULL ");
            tet_result(TET_FAIL);
            return;
        }

    } else {
        tet_infoline(" pam_get_item failed ");
        tet_result(TET_FAIL);
        return;
    }

    input_passwd = passwd;
    retval = pam_authenticate(pamh, 0);

    if (retval != PAM_USER_UNKNOWN) {

        tet_printf(" pam_authenticate returned %s ",
                                      pam_strerror(pamh, retval));
        tet_infoline(" It was expected to return PAM_USER_UNKNOWN");

        tet_result(TET_FAIL);
        return;
    }

    tet_result(TET_PASS);
    return;
}


/*
 *   login_conv():
 *        This routine is called to get the username and password
 *        required for authenticating the user. Currently the
 *        username and password are got from the configuration files
 *        to avoid user intervention.
 */

private int
login_conv(int num_msg,
           const struct pam_message **m,
           struct pam_response **r,
           void *appdata_ptr)
{
    struct  pam_response *reply;
    int     count;
    char    buf[PAM_MAX_RESP_SIZE];
    char    *temp;

    reply = (struct pam_response *)
                calloc(num_msg, sizeof(struct pam_response));

    if (reply == NULL) {
        return PAM_BUF_ERR;
    }

    for (count = 0; count < num_msg; count++) {
        reply[count].resp_retcode = 0;
        reply[count].resp = NULL;

        switch (m[count]->msg_style) {

            case PAM_PROMPT_ECHO_OFF:

                reply[count].resp = strdup(input_passwd);
                break;

            case PAM_PROMPT_ECHO_ON:

                reply[count].resp = strdup(user);
                break;

            default:
                break;
        }
    }

    *r = reply;
    return PAM_SUCCESS;
}


/*
 *  delete_tests():
 *       This routine is called to stop execution of the test cases
 *       in case of initialization errors.
 */

private void
delete_tests(char *msg)
{
    int   count;

    for (count = 1; count <= NUM_TESTS; count++) {
        tet_delete(count, msg);
    }

    return;
}
