/*
 * Copyright (c) 2002-2003 WIPRO TECHNOLOGIES 
 * Developed for LSB-PAM project by WIPRO TECHNOLOGIES
 */

/*
 * Project: LSB-PAM
 *
 * File: tset/LSB.pam/testcases/pam_chauthtok/pam_chauthtok.c
 * This file contains the test cases for testing the interface
 * pam_chauthtok.
 *
 * Author(s) : Rohit R <rohit.raveendran@wipro.com>, WIPRO TECHNOLOGIES
 *             Laavanya K R <laavanya.gopalan@wipro.com>, WIPRO TECHNOLOGIES
 *
 * Modifications:
 *
 */


#include <stdio.h>
#include <security/pam_appl.h>
#include <security/pam_misc.h>

#include <std.h>
#include <tet_api.h>

/*******************************************************************/

#define NUM_TESTS   8 /* To be updated when number of tests are modified */
#define NUM_OF_TRIES  20

private void tc1();
private void tc2();
private void tc3();
private void tc4();
private void tc5();
private void tc6();
private void tc7();
private void tc8();

private void prepare_tests();
private void clean_tests();

public  void (*tet_startup)() = prepare_tests;
public  void (*tet_cleanup)() = clean_tests;

private char *conf_file;
private char *user;

private char *passwd;
private char *input_passwd;
private char *new_passwd;
private char *tmp_passwd;
private char *temp_passwd;

private int  login_count;
private int  test_count;
private int  incr_test;

private char echo[256];

private pam_handle_t *pamh;
private struct pam_conv pamc;

public  struct tet_testlist tet_testlist[] = {
        { tc1, 1 },
        { tc2, 2 },
        { tc3, 3 },
        { tc4, 4 },
        { tc5, 5 },
        { tc6, 6 },
        { tc7, 7 },
        { tc8, 8 },
        { NULL, 0 }
};

private int login_conv(int num_msg,
                       const struct pam_message **msg,
                       struct pam_response **response,
                       void *appdata_ptr);

private void delete_tests(char *msg);
private int restore_passwd(void);

/*******************************************************************/

private void
prepare_tests()
{
    int    retval;
    char   *str = "123";

    tet_infoline(" starting tests pam_chauthtok ");

    pamh = NULL;
    pamc.conv = &login_conv;
    pamc.appdata_ptr = NULL;

    conf_file = tet_getvar("LSB_PAM_CONF_FILE");

    if (conf_file == NULL) {
        delete_tests(" tetexec.cfg parameter LSB_PAM_CONF_FILE unset");
        return;
    }

    user = tet_getvar("LSB_PAM_USER");

    if (user == NULL) {
        delete_tests(" tetexec.cfg parameter LSB_PAM_USER unset");
        return;
    }

    passwd = tet_getvar("LSB_PAM_USER_PASSWD");

    if (passwd == NULL) {
        delete_tests(" tetexec.cfg parameter LSB_PAM_USER_PASSWD unset");
        return;
    }

    retval = pam_start(conf_file, user, &pamc, &pamh);

    if (retval != PAM_SUCCESS) {
        sprintf(echo," pam_start returned %s when expected to succeed",
                                pam_strerror(NULL, retval));
        delete_tests(echo);
        return;
    }

    login_count = 0;
    input_passwd = passwd;
    retval = pam_authenticate(pamh, 0);

    if (retval != PAM_SUCCESS) {
        sprintf(echo," pam_authenticate returned %s when expected to succeed",
                                pam_strerror(NULL, retval));
        delete_tests(echo);
        return;
    }

    tmp_passwd = calloc(1, strlen(passwd) + strlen(str) + 5);

    if (tmp_passwd == NULL) {
        delete_tests(" calloc failed : deleting testcases ");
        return;
    }

    strncpy(tmp_passwd, str, strlen(str));
    strncat(tmp_passwd, passwd, strlen(passwd));

    return;
}


private void
clean_tests()
{
    if (pamh) {
        pam_end(pamh,0);
    }

    if (tmp_passwd) {
        free(tmp_passwd);
    }

    tet_infoline(" end of tests pam_chauthtok");
    return;
}


/*
 *    tc1():
 *        This test case checks the interface pam_chauthtok. 
 *        The expected return value is PAM_SUCCESS.
 */

private void
tc1()
{
    int    retval;

    login_count = 0;
    test_count = 0;
    incr_test = 0;

    input_passwd = passwd;
    new_passwd = tmp_passwd;

    retval = pam_chauthtok(pamh, 0);

    if (retval != PAM_SUCCESS) {

        tet_printf(" pam_chauthtok returned %s when expected to succeed",
                                pam_strerror(pamh, retval));
        tet_result(TET_FAIL);
        return;
    }

    if (PAM_SUCCESS != restore_passwd()) {
        delete_tests(" restore_passwd failed ");
        tet_result(TET_FAIL);
        return;
    }

    tet_result(TET_PASS);
    return;
}


/*
 *    tc2():
 *        This test case checks the interface pam_chauthtok with 
 *        flag PAM_DISALLOW_NULL_AUTHTOK and 
 *        
 *        1. correct password : expected return value is PAM_SUCCESS
 *        2. empty password : expected return value is PAM_AUTH_ERR.
 */

private void
tc2()
{
    int    retval;

    login_count = 0;
    test_count = 0;
    incr_test = 0;

    input_passwd = passwd;
    new_passwd = tmp_passwd;

    retval = pam_chauthtok(pamh, PAM_DISALLOW_NULL_AUTHTOK);

    if (retval != PAM_SUCCESS) {
        tet_infoline(" pam_chauthtok failed with correct password and");
        tet_infoline(" PAM_DISALLOW_NULL_AUTHTOK when expected to succeed.");
        tet_printf(" It returned %s", pam_strerror(pamh, retval));

        tet_result(TET_FAIL);
        return;
    }

    /* Set the password to the original one */
    if (PAM_SUCCESS != restore_passwd()) {
        delete_tests(" restore_passwd failed ");
        tet_result(TET_FAIL);
        return;
    }

    input_passwd = "\0";
    login_count = 0;
    test_count = 0;
    incr_test = 0;

    retval = pam_chauthtok(pamh, PAM_DISALLOW_NULL_AUTHTOK);

    if (retval != PAM_AUTH_ERR) {
        tet_infoline(" pam_chauthtok expected to return PAM_AUTH_ERR with ");
        tet_infoline(" empty password and flag PAM_DISALLOW_NULL_AUTHTOK.");
        tet_printf(" It returned %s", pam_strerror(pamh, retval));

        tet_result(TET_FAIL);
        return;
    }

    tet_result(TET_PASS);
    return;
}


/*
 *    tc3():
 *        This test case checks the interface pam_chauthtok with 
 *        a NULL pam handle. The return value should not be PAM_SUCCESS.
 */
private void
tc3()
{
    int    retval;

    login_count = 0;
    test_count = 0;
    incr_test = 0;

    input_passwd = passwd;

    retval = pam_chauthtok(NULL, 0);

    if (retval == PAM_SUCCESS) {

        tet_infoline(" pam_chauthtok returned PAM_SUCCESS for NULL pamh");
        tet_infoline(" It was expected not to succeed");
        tet_result(TET_FAIL);
        return;
    }

    tet_result(TET_PASS);
    return;
}


/*
 *    tc4():
 *        This test case checks the interface pam_chauthtok with 
 *        wrong password. Expected return value is PAM_AUTH_ERR. 
 */

private void
tc4()
{
    int    retval;
    char   *str = "11"; /* String appended to get invalid passwd */

    input_passwd = calloc(1, strlen(passwd) + strlen(str) + 5);

    if (input_passwd == NULL) {
        tet_infoline(" calloc failed : returning from test case ");
        tet_result(TET_FAIL);
        return;
    }

    strncpy(input_passwd, str, strlen(str));
    strncat(input_passwd, passwd, strlen(passwd));

    login_count = 0;
    test_count = 0;
    incr_test = 0;

    retval = pam_chauthtok(pamh, 0);
    free(input_passwd);
    input_passwd = NULL;

    if (retval == PAM_SUCCESS) {
        tet_infoline(" pam_chauthtok expected to fail with invalid password");

        tet_result(TET_FAIL);
        return;
    }

    tet_result(TET_PASS);
    return;
}


/*
 *    tc5():
 *        This test case checks the interface pam_chauthtok with 
 *        wrong password till the return value is PAM_MAXTRIES.
 */

private void
tc5()
{
/* This test does not reflect a requirement of the LSB 2.0 specification */
	tet_result(TET_NOTINUSE);


/*     int    retval; */
/*     int    count = NUM_OF_TRIES; */
/*     int    result = TET_FAIL; */
/*     char   *str = "11"; */

/*     input_passwd = calloc(1, strlen(passwd) + strlen(str) + 5); */

/*     if (input_passwd == NULL) { */
/*         tet_infoline(" calloc failed : returning from test case "); */
/*         tet_result(TET_FAIL); */
/*         return; */
/*     } */

/*     strncpy(input_passwd, str, strlen(str)); */
/*     strncat(input_passwd, passwd, strlen(passwd)); */

/*     while (count) { */
/*         login_count = 0; */
/*         test_count = 0; */
/*         incr_test = 0; */

/*         retval = pam_chauthtok(pamh, 0); */

/*         if (retval == PAM_MAXTRIES ) { */
/*             result = TET_PASS; */
/*             break; */
/*         } */

/*         count--; */
/*     } */

/*     free(input_passwd); */
/*     tet_result(result); */
/*     return; */
}


/*
 *    tc6():
 *        This test case checks the interface pam_chauthtok by giving 
 *        different passwords while updating the password. 
 *        The expected return value is PAM_AUTHTOK_ERR.
 *        PAM_AUTHTOK_RECOVER_ERR is also currently accepted but is
 *        deprecated.
 */

private void
tc6()
{
    int    retval;

    login_count = 0;
    test_count = 0;
    incr_test = 1;

    input_passwd = passwd;
    new_passwd = tmp_passwd;

    retval = pam_chauthtok(pamh, 0);

    if (retval == PAM_AUTHTOK_RECOVER_ERR) 
		{
			tet_infoline("PAM_AUTHTOK_RECOVER_ERR is deprecated for this situation and in the future may cause a failure of this testcase");
		}
		else if (retval != PAM_AUTHTOK_ERR) {

			if (retval==0)
			{
				/* Reset password if we accidentally suceeded */
				input_passwd = tmp_passwd;
				new_passwd = passwd;
				retval = pam_chauthtok(pamh, 0);
				if (retval!=0)
				{
					tet_printf("Failed to restore password %s",
										 pam_strerror(pamh, retval));
				}
			}

			tet_printf(" pam_chauthtok returned %s ", pam_strerror(pamh, retval));
			tet_infoline(" It was expected to return PAM_AUTHTOK_RECOVER_ERR ");
			tet_result(TET_FAIL);
			return;
    }

    tet_result(TET_PASS);
    return;
}


/*
 *    tc7():
 *        This test case checks the interface pam_chauthtok by giving 
 *        empty password while updating the password. 
 *        The expected return value is PAM_AUTHTOK_ERR.
 */

private void
tc7()
{
    int    retval;

		/* Test has been disabled because empty passwords are valid
			 according to the LSB spec */
		tet_result(TET_NOTINUSE);
		return;

    login_count = 0;
    test_count = 0;
    incr_test = 0;

    input_passwd = passwd;
    new_passwd = "";

    retval = pam_chauthtok(pamh, 0);

    if (retval != PAM_AUTHTOK_ERR) {

        tet_printf(" pam_chauthtok returned %s ", pam_strerror(pamh, retval));
        tet_infoline(" It was expected to return PAM_AUTHTOK_ERR ");
        tet_result(TET_FAIL);
        return;
    }

    tet_result(TET_PASS);
    return;
}


/*
 *    tc8():
 *        This test case checks the interface pam_chauthtok after locking
 *        the password database file. 
 *        The expected return value is PAM_AUTHTOK_LOCK_BUSY.
 */

private void
tc8()
{
    int    retval;
    int    count = 0;

    while(((retval = lckpwdf()) != 0) && count < 20) {
                usleep(1000);
                count++;
    }

    if (retval != 0) {
        tet_infoline(" Failed to get the password database lock");
        tet_result(TET_FAIL);
        return;
    }


    login_count = 0;
    test_count = 0;
    incr_test = 0;

    input_passwd = passwd;
    new_passwd = tmp_passwd;
   
    retval = pam_chauthtok(pamh, 0);
    ulckpwdf();

    if (retval != PAM_AUTHTOK_LOCK_BUSY) {

        tet_printf(" pam_chauthtok returned %s ", pam_strerror(pamh, retval));
        tet_infoline(" It was expected to return PAM_AUTHTOK_LOCK_BUSY");
        tet_result(TET_FAIL);
        return;
    }

    tet_result(TET_PASS);
    return;
}


/*
 *   login_conv():
 *        This routine is called to get the username and password
 *        required for authenticating the user. Currently the 
 *        username and password are got from the configuration files
 *        to avoid user intervention.
 */

private int
login_conv(int num_msg,
           const struct pam_message **m,
           struct pam_response **r,
           void *appdata_ptr)
{
    struct  pam_response *reply;
    int     count;
    char    buf[PAM_MAX_RESP_SIZE];
    char    *temp;

    reply = (struct pam_response *)
                calloc(num_msg, sizeof(struct pam_response));

    if (reply == NULL) {
        return PAM_BUF_ERR;
    }

    for (count = 0; count < num_msg; count++) {
        reply[count].resp_retcode = 0;
        reply[count].resp = NULL; 

        switch (m[count]->msg_style) {

            case PAM_PROMPT_ECHO_OFF:

                if (test_count != 0) {
                    temp_passwd = "temp_passwd";
                }

                if (login_count == 0) {
                    reply[count].resp = strdup(input_passwd);

                } else {
                    if (test_count == 2) {
                        reply[count].resp = strdup(temp_passwd);
                    } else {
                        reply[count].resp = strdup(new_passwd);
                    }
                }
								tet_printf("Sending password %s", reply[count].resp);

		login_count++;

                if (incr_test != 0) {
                    test_count++;
                }

		break;

            case PAM_PROMPT_ECHO_ON:

                reply[count].resp = strdup(user);
                break;

            default:
                break;
        }
    }

    *r = reply;
    return PAM_SUCCESS;
}


/*
 *    restore_passwd():
 *        This test case checks resets the passwd back to the original one. 
 *        The expected return value is PAM_SUCCESS.
 */

private int
restore_passwd()
{
    int    retval;

    login_count = 0;
    
    new_passwd = passwd;
    input_passwd = tmp_passwd;

    retval = pam_chauthtok(pamh, 0);

    if (retval != PAM_SUCCESS) {
        tet_infoline(" restore_passwd() failed to restore original password");
        tet_printf(" pam_chauthtok returned %s when expected to succeed",
                                pam_strerror(pamh, retval));
    }

    return retval;
}


/*
 *  delete_tests():
 *       This routine is called to stop execution of the test cases
 *       in case of initialization errors.
 */

private void
delete_tests(char *msg)
{
    int   count;

    for (count = 1; count <= NUM_TESTS; count++) {
        tet_delete(count, msg);
    }

    return;
}
