/*
 * Copyright (c) 2002-2003 WIPRO TECHNOLOGIES 
 * Developed for LSB-PAM project by WIPRO TECHNOLOGIES
 */

/*
 * Project: LSB-PAM
 *
 * File: tset/LSB.pam/testcases/pam_close_session/pam_close_session.c
 * This file contains the test cases for testing the interface
 * pam_close_session.
 *
 * Author(s) : Rohit R <rohit.raveendran@wipro.com>, WIPRO TECHNOLOGIES
 *             Laavanya K R <laavanya.gopalan@wipro.com>, WIPRO TECHNOLOGIES
 *
 * Modifications:
 *
 */


#include <stdio.h>
#include <security/pam_appl.h>
#include <security/pam_misc.h>

#include <std.h>
#include <tet_api.h>

/*******************************************************************/

#define NUM_TEST_CASES   3 /* To be updated when number of tests are modified */

private void tc1();
private void tc2();
private void tc3();

private void prepare_tests();
private void clean_tests();

public  void (*tet_startup)() = prepare_tests;
public  void (*tet_cleanup)() = clean_tests;

private char *conf_file;
private char *user;
private char *passwd;
private char *input_passwd;

private char echo[256];

private pam_handle_t *pamh;
private struct pam_conv pamc;

public  struct tet_testlist tet_testlist[] = {
        { tc1, 1 },
        { tc2, 2 },
        { tc3, 3 },
        { NULL, 0 }
};

private int login_conv(int num_msg,
                       const struct pam_message **msg,
                       struct pam_response **response,
                       void *appdata_ptr);

private void delete_tests(char *msg);

/*******************************************************************/

private void
prepare_tests()
{
    int    retval;

    tet_infoline(" starting tests pam_close_session ");

    pamc.conv = &login_conv;
    pamc.appdata_ptr = NULL;

    conf_file = tet_getvar("LSB_PAM_CONF_FILE");

    if (conf_file == NULL) {
        delete_tests(" tetexec.cfg parameter LSB_PAM_CONF_FILE unset");
        return;
    }

    user = tet_getvar("LSB_PAM_USER");

    if (user == NULL) {
        delete_tests(" tetexec.cfg parameter LSB_PAM_USER unset");
        return;
    }

    passwd = tet_getvar("LSB_PAM_USER_PASSWD");

    if (passwd == NULL) {
        delete_tests(" tetexec.cfg parameter LSB_PAM_USER_PASSWD unset");
        return;
    }

    retval = pam_start(conf_file, user, &pamc, &pamh);

    if (retval != PAM_SUCCESS) {
        sprintf(echo," pam_start returned %s, when expected to succeed.",
                                pam_strerror(NULL, retval));
        delete_tests(echo);

        return;
    }

    input_passwd = passwd;

    retval = pam_authenticate(pamh, 0);

    if (retval != PAM_SUCCESS) {

        sprintf(echo," pam_authenticate returned %s, when expected to succeed.",
                                pam_strerror(pamh, retval));
        delete_tests(echo);

        return;
    }

    return;
}


private void
clean_tests()
{
    tet_infoline(" end of tests pam_close_session ");

    if(pamh){
        pam_end(pamh,0);
    }

    return;
}


/*
 *    tc1():
 *        This test case checks the interface pam_close_session.
 *        The expected return value is PAM_SUCCESS.
 */

private void
tc1()
{
    int    retval;

    retval = pam_open_session(pamh, PAM_SILENT);

    if (retval != PAM_SUCCESS) {
        sprintf(echo," pam_open_session returned %s, when expected to succeed.",
                                pam_strerror(pamh, retval));
        delete_tests(echo);

        tet_result(TET_FAIL);
        return;
    }

    retval = pam_close_session(pamh, PAM_SILENT);

    if (retval != PAM_SUCCESS) {
        sprintf(echo," pam_close_session returned %s, when expected to succeed.",
                                pam_strerror(pamh, retval));
        delete_tests(echo);

        tet_result(TET_FAIL);
        return;
    }

    tet_result(TET_PASS);
    return;
}


/*
 *    tc2():
 *        This test case checks the interface pam_close_session with
 *        PAM_USER item as empty. Expected return value is PAM_SESSION_ERR.
 */

private void
tc2()
{
    int     retval;
    char    *value;
    char    *orig_value = NULL; 

    /* Opening a session */
    retval = pam_open_session(pamh, PAM_SILENT);

    if (retval != PAM_SUCCESS) {
        sprintf(echo," pam_open_session returned %s, when expected to succeed",
                                pam_strerror(pamh, retval));
        delete_tests(echo);

        tet_result(TET_FAIL);
        return;
    }

    retval = pam_get_item(pamh, PAM_USER, (const void **)&value);

    if (retval != PAM_SUCCESS) {
        tet_printf(" pam_get_item returned %s, when expected to succeed", 
                                pam_strerror(pamh, retval));

        retval = pam_close_session(pamh, PAM_SILENT);

	if (retval != PAM_SUCCESS) {
            tet_printf(" pam_close_session failed returning %s",
                                pam_strerror(pamh, retval));
        }

        tet_result(TET_FAIL);
        return;
    }

    /* Copying the retrieved pam item value for restoring later */
    if(value) {
        orig_value = strdup(value);

        if(orig_value == NULL) {
            tet_infoline(" strdup failed");

            retval = pam_close_session(pamh, PAM_SILENT);

	    if (retval != PAM_SUCCESS) {
                tet_printf(" pam_close_session failed, returning %s\n",
                                pam_strerror(pamh, retval));
            }

	    tet_result(TET_FAIL);
            return;
        }
    }
 
    /* Changing item type value */
    retval = pam_set_item(pamh, PAM_USER, "");   

    if (retval != PAM_SUCCESS) {
        tet_printf(" pam_set_item returned %s, when expected to succeed\n",
                                pam_strerror(pamh, retval));

        retval = pam_close_session(pamh, PAM_SILENT);

	if (retval != PAM_SUCCESS) {
            tet_printf(" pam_close_session failed, returning %s\n",
                                pam_strerror(pamh, retval));
        }

        if (orig_value != NULL) {
            free(orig_value);
            orig_value = NULL;
        }

        tet_result(TET_FAIL);
        return;
    }

    retval = pam_close_session(pamh, PAM_SILENT);

    if (retval != PAM_SESSION_ERR) {
        tet_printf(" pam_close_session returned %s instead of PAM_SESSION_ERR",
                                pam_strerror(pamh, retval));

        if (orig_value) {
            free(orig_value);
            orig_value = NULL;
        }

        tet_result(TET_FAIL);
        return;
    }

    /* Restoring item type value through pam_set_item */
    retval = pam_set_item(pamh, PAM_USER, orig_value);

    if (retval != PAM_SUCCESS) {
        tet_printf(" pam_set_item returned %s, when expected to succeed\n",
                                pam_strerror(pamh, retval));

        retval = pam_close_session(pamh, PAM_SILENT);

	if (retval != PAM_SUCCESS) {
            tet_printf(" pam_close_session failed with %s\n",
                                pam_strerror(pamh, retval));
        }

        if (orig_value != NULL) {
            free(orig_value);
            orig_value = NULL;
        }

        tet_result(TET_FAIL);
        return;
    }

    if (orig_value) {
        free(orig_value);
        orig_value = NULL;
    }

    retval = pam_close_session(pamh, PAM_SILENT);

    if (retval != PAM_SUCCESS) {
        tet_printf(" pam_close_session failed with %s\n",
                                pam_strerror(pamh, retval));

        tet_result(TET_FAIL);
        return;
    }

    tet_result(TET_PASS);
    return;
}


/*
 *    tc3():
 *        This test case checks the interface pam_close_session with pamh
 *        as NULL. The return value should not be  PAM_SUCCESS.
 */

private void
tc3()
{
    int    retval;

    retval = pam_open_session(pamh, PAM_SILENT);

    if (retval != PAM_SUCCESS) {
        sprintf(echo," pam_open_session returned %s, when expected to succeed",
                                pam_strerror(pamh, retval));
        delete_tests(echo);

        tet_result(TET_FAIL);
        return;
    }

    retval = pam_close_session(NULL, PAM_SILENT);

    if (retval == PAM_SUCCESS) {
        tet_infoline(" pam_close_session returned PAM_SUCCESS with NULL pamh");
        tet_infoline(" It was expected to fail");

        tet_result(TET_FAIL);
        return;
    }

    retval = pam_close_session(pamh, PAM_SILENT);

    if (retval != PAM_SUCCESS) {
        tet_printf(" pam_close_session returned %s, when expected to succeed\n",
                                pam_strerror(pamh, retval));

        tet_result(TET_FAIL);
        return;
    }

    tet_result(TET_PASS);
    return;
}

/*
 *   login_conv():
 *        This routine is called to get the username and password
 *        required for authenticating the user. Currently the
 *        username and password are got from the configuration files
 *        to avoid user intervention.
 */

private int
login_conv(int num_msg,
           const struct pam_message **m,
           struct pam_response **r,
           void *appdata_ptr)
{
    struct  pam_response *reply;
    int     count;
    char    buf[PAM_MAX_RESP_SIZE];
    char    *temp;

    reply = (struct pam_response *)
                calloc(num_msg, sizeof(struct pam_response));

    if (reply == NULL) {
        return PAM_BUF_ERR;
    }
	
    for (count = 0; count < num_msg; count++) {
        reply[count].resp_retcode = 0;
        reply[count].resp = NULL;

        switch (m[count]->msg_style) {

            case PAM_PROMPT_ECHO_OFF:

                reply[count].resp = strdup(input_passwd);
                break;

            case PAM_PROMPT_ECHO_ON:

                reply[count].resp = strdup(user);
                break;

            default:
                break;
        }
    }

    if (reply) {
        *r = reply;
    }

    return PAM_SUCCESS;
}


/*
 *  delete_tests():
 *       This routine is called to stop execution of the test cases
 *       in case of initialization errors.
 */

private void
delete_tests(char *msg)
{
    int   count;

    for (count = 1; count <= NUM_TEST_CASES; count++) {
        tet_delete(count, msg);
    }

    return;
}
