/*
 * Copyright (c) 2002-2003 WIPRO TECHNOLOGIES 
 * Developed for LSB-PAM project by WIPRO TECHNOLOGIES
 */

/*
 * Project: LSB-PAM
 *
 * File: tset/LSB.pam/testcases/pam_setcred/pam_setcred.c
 * This file contains the test cases for testing the interface
 * pam_setcred.
 *
 * Author(s) : Rohit R <rohit.raveendran@wipro.com>, WIPRO TECHNOLOGIES
 *             Laavanya K R <laavanya.gopalan@wipro.com>, WIPRO TECHNOLOGIES
 *
 * Modifications:
 *
 */

#include <stdio.h>
#include <security/pam_appl.h>
#include <security/pam_misc.h>

#include <std.h>
#include <tet_api.h>

/*******************************************************************/

#define NUM_TESTS   2 /* To be updated when number of tests are modified */

private void tc1();
private void tc2();

private void prepare_tests();
private void clean_tests();

public  void (*tet_startup)() = prepare_tests;
public  void (*tet_cleanup)() = clean_tests;

private char *conf_file;
private char *user;
private char *passwd;
private char *input_passwd;

private pam_handle_t *pamh;
private struct pam_conv pamc;

private char echo[256];

struct mapping {
        int flag;
        char *desc;
};

private struct mapping flags[] = {
        { PAM_ESTABLISH_CRED, "PAM_ESTABLISH_CRED" },
        { PAM_DELETE_CRED, "PAM_DELETE_CRED" },
        { PAM_REINITIALIZE_CRED, "PAM_REINITIALIZE_CRED" },
        { PAM_REFRESH_CRED, "PAM_REFRESH_CRED" }
};

public struct tet_testlist tet_testlist[] = {
        { tc1, 1 },
        { tc2, 2 },
        { NULL, 0 }
};

private int login_conv(int num_msg,
                       const struct pam_message **msg,
                       struct pam_response **response,
                       void *appdata_ptr);

private void delete_tests(char *msg);

/*******************************************************************/

private void
prepare_tests()
{
    int    retval;

    tet_infoline(" starting tests pam_setcred ");

    pamh = NULL;
    pamc.conv = &login_conv;
    pamc.appdata_ptr = NULL;

    conf_file = tet_getvar("LSB_PAM_CONF_FILE");

    if (conf_file == NULL) {
        delete_tests(" tetexec.cfg parameter LAB_PAM_CONF_FILE unset");
        return;
    }

    user = tet_getvar("LSB_PAM_USER");

    if (user == NULL) {
        delete_tests(" tetexec.cfg parameter LAB_PAM_USER unset");
        return;
    }

    passwd = tet_getvar("LSB_PAM_USER_PASSWD");

    if (passwd == NULL) {
        delete_tests(" tetexec.cfg parameter LSB_PAM_USER_PASSWD unset");
        return;
    }

    retval = pam_start(conf_file, user, &pamc, &pamh);

    if (retval != PAM_SUCCESS) {
        sprintf(echo," pam_start returned %s when expected to succeed",
                                pam_strerror(NULL, retval));
        delete_tests(echo);
        return;
    }

    input_passwd = passwd;
    retval = pam_authenticate(pamh, 0);

    if (retval != PAM_SUCCESS) {
        sprintf(echo," pam_authenticate returned %s when expected to succeed",
                                pam_strerror(NULL, retval));
        delete_tests(echo);
        return;
    }

    return;
}


private void
clean_tests()
{
    if (pamh) {
        pam_end(pamh,0);
    }

    tet_infoline(" end of tests pam_setcred");
    return;
}


/*
 * tc1() :
 *     This test case checks the interface pam_setcred with valid
 *     parameters. The expected return value is PAM_SUCCESS.
 */

private void
tc1()
{
    int    retval;
    int    count;
    int    index;


    count = sizeof(flags) / sizeof(struct mapping);

    for (index = 0; index < count; index++) {
        retval = pam_setcred(pamh, flags[index].flag);

        if (retval != PAM_SUCCESS) {
            tet_printf(" pam_setcred returned %s for flag %s ",
                        pam_strerror(pamh, retval), flags[index].desc);
            tet_infoline(" It was expected to return PAM_SUCCESS");
            tet_result(TET_FAIL);
            return;
        }
    }

    tet_result(TET_PASS);
    return;
}


/*
 *    tc2() :
 *        This test case tests the interface pam_setcred with NULL for
 *        pam handle. The return value should not be PAM_SUCCESS.
 */

private void
tc2()
{
    int    retval;
    int    count;
    int    index;


    count = sizeof(flags) / sizeof(struct mapping);

    for (index = 0; index < count; index++) {

        retval = pam_setcred(NULL, flags[index].flag);

        if (retval == PAM_SUCCESS) {
            tet_printf(" pam_setcred returned %s for %s with NULL pamh",
                        pam_strerror(pamh, retval), flags[index].desc);
            tet_infoline(" It was expected to fail");
            tet_result(TET_FAIL);
            return;

        }
    }

    tet_result(TET_PASS);
    return;
}


/*
 *   login_conv():
 *        This routine is called to get the username and password
 *        required for authenticating the user. Currently the
 *        username and password are got from the configuration files
 *        to avoid user intervention.
 */

private int
login_conv(int num_msg,
           const struct pam_message **m,
           struct pam_response **r,
           void *appdata_ptr)
{
    struct  pam_response *reply;
    int     count;
    char    buf[PAM_MAX_RESP_SIZE];
    char    *temp;

    reply = (struct pam_response *)
                calloc(num_msg, sizeof(struct pam_response));

    if (reply == NULL) {
        return PAM_BUF_ERR;
    }

    for (count = 0; count < num_msg; count++) {
        reply[count].resp_retcode = 0;
        reply[count].resp = NULL;

        switch (m[count]->msg_style) {

            case PAM_PROMPT_ECHO_OFF:

                reply[count].resp = strdup(input_passwd);
                break;

            case PAM_PROMPT_ECHO_ON:

                reply[count].resp = strdup(user);
                break;

            default:
                break;
        }
    }

    *r = reply;
    return PAM_SUCCESS;
}



/*
 *  delete_tests():
 *       This routine is called to stop execution of the test cases
 *       in case of initialization errors.
 */

private void
delete_tests(char *msg)
{
    int    count;

    for (count = 1; count <= NUM_TESTS; count++) {
        tet_delete(count, msg);
    }

    return;
}
